# -*- coding: utf-8 -*-

from io import StringIO
from subprocess import call
from os.path import basename, splitext
import os
import shutil
import sys
import traceback

from DisplayCAL.meta import name
from DisplayCAL.util_os import relpath, safe_glob, which

recordfile_name = "INSTALLED_FILES"

if sys.stdout and hasattr(sys.stdout, "isatty") and not sys.stdout.isatty():
    sys.stdout = StringIO()

if sys.platform == "win32":
    try:
        create_shortcut
    # this function is only available within bdist_wininst installers
    except NameError:
        try:
            from pythoncom import (
                CoCreateInstance,
                CLSCTX_INPROC_SERVER,
                IID_IPersistFile,
            )
            from win32com.shell import shell
            import win32con
        except ImportError:

            def create_shortcut(*args):
                pass

        else:

            def create_shortcut(*args):
                shortcut = CoCreateInstance(
                    shell.CLSID_ShellLink,
                    None,
                    CLSCTX_INPROC_SERVER,
                    shell.IID_IShellLink,
                )
                shortcut.SetPath(args[0])
                shortcut.SetDescription(args[1])
                if len(args) > 3:
                    shortcut.SetArguments(args[3])
                if len(args) > 4:
                    shortcut.SetWorkingDirectory(args[4])
                if len(args) > 5:
                    shortcut.SetIconLocation(args[5], args[6] if len(args) > 6 else 0)
                shortcut.SetShowCmd(win32con.SW_SHOWNORMAL)
                shortcut.QueryInterface(IID_IPersistFile).Save(args[2], 0)

    try:
        directory_created
    # this function is only available within bdist_wininst installers
    except NameError:

        def directory_created(path):
            pass

    try:
        file_created
    # this function is only available within bdist_wininst installers
    except NameError:
        try:
            import win32api
        except ImportError:

            def file_created(path):
                pass

        else:

            def file_created(path):
                if os.path.exists(recordfile_name):
                    installed_files = []
                    if os.path.exists(recordfile_name):
                        recordfile = open(recordfile_name, "r")
                        installed_files.extend(line.rstrip("\n") for line in recordfile)
                        recordfile.close()
                    try:
                        path.encode("ASCII")
                    except (UnicodeDecodeError, UnicodeEncodeError):
                        # the contents of the record file used by distutils
                        # must be ASCII GetShortPathName allows us to avoid
                        # any issues with encoding because it returns the
                        # short path as 7-bit string (while still being a
                        # valid path)
                        path = win32api.GetShortPathName(path)
                    installed_files.append(path)
                    recordfile = open(recordfile_name, "w")
                    recordfile.write("\n".join(installed_files))
                    recordfile.close()

    try:
        get_special_folder_path
    # this function is only available within bdist_wininst installers
    except NameError:
        try:
            from win32com.shell import shell, shellcon
        except ImportError:

            def get_special_folder_path(csidl_string):
                pass

        else:

            def get_special_folder_path(csidl_string):
                return shell.SHGetSpecialFolderPath(
                    0, getattr(shellcon, csidl_string), 1
                )


def postinstall_macos(prefix=None):
    """Do postinstall actions for macOS."""
    # TODO: implement
    pass


def postinstall_windows(prefix):
    """Do postinstall actions for Windows."""
    if prefix is None:
        # assume we are running from bdist_wininst installer
        modpath = os.path.dirname(os.path.abspath(__file__))
    else:
        # assume we are running from source dir,
        # or from install dir
        modpath = prefix

    if not os.path.exists(modpath):
        print("warning - '{}' not found".format(modpath.encode("MBCS", "replace")))
        return

    if os.path.exists(recordfile_name):
        irecordfile_name = os.path.join(modpath, "INSTALLED_FILES")
        with open(irecordfile_name, "w"):  # touch create the file
            pass
        file_created(irecordfile_name)
        shutil.copy2(recordfile_name, irecordfile_name)

    mainicon = os.path.join(modpath, "theme", "icons", f"{name}.ico")
    if not os.path.exists(mainicon):
        print("warning - '{}' not found".format(icon.encode("MBCS", "replace")))
        return

    try:
        startmenu_programs_common = get_special_folder_path("CSIDL_COMMON_PROGRAMS")
        startmenu_programs = get_special_folder_path("CSIDL_PROGRAMS")
        startmenu_common = get_special_folder_path("CSIDL_COMMON_STARTMENU")
        startmenu = get_special_folder_path("CSIDL_STARTMENU")
    except OSError:
        traceback.print_exc()
        return

    filenames = [
        filename
        for filename in safe_glob(os.path.join(sys.prefix, "Scripts", f"{name}*"))
        if not filename.endswith("-script.py")
        and not filename.endswith("-script.pyw")
        and not filename.endswith(".manifest")
        and not filename.endswith(".pyc")
        and not filename.endswith(".pyo")
        and not filename.endswith("_postinstall.py")
    ] + ["LICENSE.txt", "README.html", "Uninstall"]
    installed_shortcuts = []
    for path in (startmenu_programs_common, startmenu_programs):
        if not path:
            continue
        grppath = os.path.join(path, name)
        if path == startmenu_programs:
            group = relpath(grppath, startmenu)
        else:
            group = relpath(grppath, startmenu_common)

        if not os.path.exists(grppath):
            try:
                os.makedirs(grppath)
            except Exception:
                # maybe insufficient privileges?
                pass

        if os.path.exists(grppath):
            print(
                ("Created start menu group '{}' in {}").format(
                    name,
                    (
                        str(path, "MBCS", "replace") if not isinstance(path, str) else path
                    ).encode("MBCS", "replace"),
                )
            )
        else:
            print(
                ("Failed to create start menu group '{}' in {}").format(
                    name,
                    (
                        str(path, "MBCS", "replace") if not isinstance(path, str) else path
                    ).encode("MBCS", "replace"),
                )
            )
            continue
        directory_created(grppath)
        for filename in filenames:
            lnkname = splitext(basename(filename))[0]
            lnkpath = os.path.join(grppath, f"{lnkname}.lnk")
            if os.path.exists(lnkpath):
                try:
                    os.remove(lnkpath)
                except Exception:
                    # maybe insufficient privileges?
                    print(
                        ("Failed to create start menu entry '{}' in {}").format(
                            lnkname,
                            (
                                str(grppath, "MBCS", "replace")
                                if not isinstance(grppath, str)
                                else grppath
                            ).encode("MBCS", "replace"),
                        )
                    )
                    continue
            if not os.path.exists(lnkpath):
                if lnkname != "Uninstall":
                    tgtpath = os.path.join(modpath, filename)
                try:
                    if lnkname == "Uninstall":
                        uninstaller = os.path.join(sys.prefix, f"Remove{name}.exe")
                        if os.path.exists(uninstaller):
                            create_shortcut(
                                uninstaller,
                                lnkname,
                                lnkpath,
                                '-u "{}-wininst.log"'.format(
                                    os.path.join(sys.prefix, name)
                                ),
                                sys.prefix,
                                os.path.join(
                                    modpath,
                                    "theme",
                                    "icons",
                                    f"{name}-uninstall.ico",
                                ),
                            )
                        else:
                            # When running from a
                            # bdist_wininst or bdist_msi
                            # installer, sys.executable
                            # points to the installer
                            # executable, not python.exe
                            create_shortcut(
                                os.path.join(sys.prefix, "python.exe"),
                                lnkname,
                                lnkpath,
                                '"{}" uninstall --record="{}"'.format(
                                    os.path.join(modpath, "setup.py"),
                                    os.path.join(modpath, "INSTALLED_FILES"),
                                ),
                                sys.prefix,
                                os.path.join(
                                    modpath,
                                    "theme",
                                    "icons",
                                    f"{name}-uninstall.ico",
                                ),
                            )
                    elif lnkname.startswith(name):
                        # When running from a
                        # bdist_wininst or bdist_msi
                        # installer, sys.executable
                        # points to the installer
                        # executable, not python.exe
                        icon = os.path.join(
                            modpath,
                            "theme",
                            "icons",
                            f"{lnkname}.ico",
                        )
                        icon = mainicon if not os.path.isfile(icon) else icon
                        if filename.endswith(".exe"):
                            exe = filename
                            args = ""
                        else:
                            exe = os.path.join(sys.prefix, "pythonw.exe")
                            args = f'"{tgtpath}"'
                        create_shortcut(
                            exe,
                            lnkname,
                            lnkpath,
                            args,
                            modpath,
                            icon,
                        )
                    else:
                        create_shortcut(tgtpath, lnkname, lnkpath, "", modpath)
                except Exception:
                    # maybe insufficient privileges?
                    print(
                        ("Failed to create start menu entry '{}' in {}").format(
                            lnkname,
                            (
                                str(grppath, "MBCS", "replace")
                                if not isinstance(grppath, str)
                                else grppath
                            ).encode("MBCS", "replace"),
                        )
                    )
                    continue
                print(
                    ("Installed start menu entry '{}' to {}").format(
                        lnkname,
                        (
                            str(group, "MBCS", "replace")
                            if not isinstance(group, str)
                            else group
                        ).encode("MBCS", "replace"),
                    )
                )
            file_created(lnkpath)
            installed_shortcuts.append(filename)
        if installed_shortcuts == filenames:
            break


def postinstall_linux(prefix=None):
    """Do postinstall actions for Linux."""
    # Linux/Unix
    if prefix is None:
        prefix = sys.prefix
    if which("touch"):
        call(["touch", "--no-create", f"{prefix}/share/icons/hicolor"])
    if which("xdg-icon-resource"):
        # print("installing icon resources...")
        # for size in [16, 22, 24, 32, 48, 256]:
        # call([
        #     "xdg-icon-resource",
        #     "install",
        #     "--noupdate",
        #     "--novendor",
        #     "--size",
        #     str(size),
        #     f"{prefix}/share/{name}/theme/icons/{size}x{size}/{name}.png"
        # ])
        call(["xdg-icon-resource", "forceupdate"])
    if which("xdg-desktop-menu"):
        # print("installing desktop menu entry...")
        # call([
        #     "xdg-desktop-menu",
        #     "install",
        #     "--novendor",
        #     f"{prefix}/share/{name}/{name}.desktop"
        # ])
        call(["xdg-desktop-menu", "forceupdate"])


def postinstall(prefix=None):
    if sys.platform == "darwin":
        postinstall_macos()
    elif sys.platform == "win32":
        postinstall_windows(prefix)
    else:
        postinstall_linux(prefix)


def postuninstall(prefix=None):
    if sys.platform == "darwin":
        # TODO: implement
        pass
    elif sys.platform == "win32":
        # nothing to do
        pass
    else:
        # Linux/Unix
        if prefix is None:
            prefix = sys.prefix
        if which("xdg-desktop-menu"):
            # print("uninstalling desktop menu entry...")
            # call(["xdg-desktop-menu", "uninstall", prefix +
            # (f"/share/applications/{name}.desktop")])
            call(["xdg-desktop-menu", "forceupdate"])
        if which("xdg-icon-resource"):
            # print("uninstalling icon resources...")
            # for size in [16, 22, 24, 32, 48, 256]:
            # call(["xdg-icon-resource", "uninstall", "--noupdate", "--size",
            # str(size), name])
            call(["xdg-icon-resource", "forceupdate"])


def main():
    prefix = None
    for arg in sys.argv[1:]:
        arg = arg.split("=")
        if len(arg) == 2:
            if arg[0] == "--prefix":
                prefix = arg[1]
    try:
        if "-remove" in sys.argv[1:]:
            postuninstall(prefix)
        else:
            postinstall(prefix)
    except Exception:
        traceback.print_exc()


if __name__ == "__main__":
    main()



# ------------------------------------------------------------------------------------ #
# GRADIENTBUTTON wxPython IMPLEMENTATION
#
# Andrea Gavana, @ 07 October 2008
# Latest Revision: 27 Dec 2012, 21.00 GMT
#
#
# TODO List
#
# 1) Anything to do?
#
#
# For all kind of problems, requests of enhancements and bug reports,
# please write to me at:
#
# andrea.gavana@gmail.com
# andrea.gavana@maerskoil.com
#
# Or, obviously, to the wxPython mailing list!!!
#
# Tags:        phoenix-port, unittest, documented, py3-port
#
# End Of Comments
# ------------------------------------------------------------------------------------ #

"""`GradientButton` mimics Windows CE mobile gradient buttons.

Description
===========

:class:`GradientButton` is another custom-drawn button class which mimics Windows CE
mobile gradient buttons, using a tri-vertex blended gradient plus some ClearType bold
font (best effect with Tahoma Bold). :class:`GradientButton` supports:

* Triple blended gradient background, with customizable colours;
* Custom colours for the "pressed" state;
* Rounded-corners buttons;
* Text-only or image+text buttons.

And a lot more. Check the demo for an almost complete review of the functionalities.

Usage
=====

Usage example::

import wx
import wx.lib.agw.gradientbutton as GB


class MyFrame(wx.Frame):

    def __init__(self, parent: wx.Window | None) -> None:
        wx.Frame.__init__(self, parent, -1, "GradientButton Demo")

        panel = wx.Panel(self, -1)

        # Create a vertical box sizer
        sizer = wx.BoxSizer(wx.VERTICAL)

        # One button without bitmap
        button_size = wx.Size(100, 50)
        sizer.Add(
            GB.GradientButton(panel, -1, None, "Hello World", size=button_size),
            0,
            wx.ALL,
            10,
        )

        # One button with bitmap
        my_bitmap = wx.Bitmap("my_bitmap.png", wx.BITMAP_TYPE_PNG)
        sizer.Add(
            GB.GradientButton(panel, -1, my_bitmap, "GradientButton", size=button_size),
            0,
            wx.ALL,
            10,
        )

        # Set the sizer for the panel
        panel.SetSizer(sizer)

        # Fit the frame to the sizer
        self.Fit()


# our normal wxApp-derived class, as usual

app = wx.App(False)

frame = MyFrame(None)
app.SetTopWindow(frame)
frame.Show()

app.MainLoop()

Supported Platforms
===================

:class:`GradientButton` has been tested on the following platforms:
  * Windows (Windows XP).


Window Styles
=============

`No particular window styles are available for this class.`


Events Processing
=================

This class processes the following events:

================= ======================================================================
Event Name        Description
================= ======================================================================
``wx.EVT_BUTTON`` Process a `wxEVT_COMMAND_BUTTON_CLICKED` event,
                    when the button is clicked.
================= ======================================================================


License And Version
===================

:class:`GradientButton` is distributed under the wxPython license.

Latest Revision: Andrea Gavana @ 27 Dec 2012, 21.00 GMT

Version 0.3
"""

from typing import Optional, Union

import wx


HOVER = 1
"""Flag used to indicate that the mouse is hovering on a :class:`GradientButton`."""
CLICK = 2
"""Flag used to indicate that the :class:`GradientButton` is on a pressed state."""


class GradientButtonEvent(wx.PyCommandEvent):
    """Event sent from :class:`GradientButton` when the button is activated.

    Args:
        eventType (int): the event type;
        eventId (int): the event identifier.
    """

    def __init__(self, eventType: int, eventId: int) -> None:
        wx.PyCommandEvent.__init__(self, eventType, eventId)
        self.isDown = False
        self.theButton: Union[None, GradientButton] = None

    def SetButtonObj(self, btn: "GradientButton") -> None:
        """Set the event object for the event.

        Args:
            btn ('GradientButton'): The button object.
        """
        self.theButton = btn

    def GetButtonObj(self) -> Union[None, "GradientButton"]:
        """Return the object associated with this event.

        Returns:
            Union[None, GradientButton]: The button object associated with this event,
                or None if no button is associated.
        """
        return self.theButton


class GradientButton(wx.Control):
    """This is the main class implementation of :class:`GradientButton`.

    Args:
        parent (GradientButton): The :class:`GradientButton` parent.
        id (int): Window identifier. A value of -1 indicates a default value.
        bitmap (Optional[wx.Bitmap]): The button bitmap (if any).
        label (str): The button text label;
        pos (wx.Point): The control position.
            A value of (-1, -1) indicates a default position, chosen by either the
            windowing system or wxPython, depending on platform.
        size (wx.Size): The control size.
            A value of (-1, -1) indicates a default size, chosen by either the
            windowing system or wxPython, depending on platform.
        style (int): The button style (unused);
        align (int): Text/bitmap alignment. wx.CENTER or wx.LEFT;
        validator (wx.Validator): The validator associated to the button;
        name (str): the button name.
    """

    def __init__(
        self,
        parent: "GradientButton",
        id: int = wx.ID_ANY,
        bitmap: Optional[wx.Bitmap] = None,
        label: str = "",
        pos: wx.Point = wx.DefaultPosition,
        size: wx.Size = wx.DefaultSize,
        style: int = wx.NO_BORDER,
        align: int = wx.CENTER,
        validator: wx.Validator = wx.DefaultValidator,
        name: str = "gradientbutton",
    ) -> None:
        super().__init__(parent, id, pos, size, style, validator, name)

        self.Bind(wx.EVT_PAINT, self.OnPaint)
        self.Bind(wx.EVT_ERASE_BACKGROUND, lambda event: None)
        self.Bind(wx.EVT_SIZE, self.OnSize)
        self.Bind(wx.EVT_LEFT_DOWN, self.OnLeftDown)
        self.Bind(wx.EVT_LEFT_UP, self.OnLeftUp)
        self.Bind(wx.EVT_LEAVE_WINDOW, self.OnMouseLeave)
        self.Bind(wx.EVT_ENTER_WINDOW, self.OnMouseEnter)
        self.Bind(wx.EVT_SET_FOCUS, self.OnGainFocus)
        self.Bind(wx.EVT_KILL_FOCUS, self.OnLoseFocus)
        self.Bind(wx.EVT_KEY_DOWN, self.OnKeyDown)
        self.Bind(wx.EVT_KEY_UP, self.OnKeyUp)
        self.Bind(wx.EVT_LEFT_DCLICK, self.OnLeftDown)

        self._mouseAction = None
        self._hasFocus = False
        self._alignment = align
        self.SetBitmapLabel(bitmap)

        self.SetLabel(label)
        self.InheritAttributes()
        self.SetInitialSize(size)

        self.SetBaseColours()

    def SetBitmapLabel(self, bitmap: Union[None, wx.Bitmap]) -> None:
        """Set the bitmap label for the button.

        Args:
            bitmap (Union[Non, wx.Bitmap]): the bitmap label to set,.
        """
        self._bitmap = bitmap
        self.Refresh()

    def SetBaseColours(
        self,
        startcolour: Union[None, wx.Colour] = None,
        foregroundcolour: Union[None, wx.Colour] = None,
    ) -> None:
        """Set the bottom, top, pressed and foreground colour.

        Args:
            startcolour (wx.Colour): based colour to be used for bottom, top and
                pressed.
            foregroundcolour (wx.Colour): colour used for the text.
        """
        if startcolour is None:
            startcolour = wx.BLACK
        if foregroundcolour is None:
            foregroundcolour = wx.WHITE

        self._bottomStartColour = startcolour
        rgba = (
            self._bottomStartColour.Red(),
            self._bottomStartColour.Green(),
            self._bottomStartColour.Blue(),
            self._bottomStartColour.Alpha(),
        )
        self._bottomEndColour = self.LightColour(self._bottomStartColour, 20)
        self._topStartColour = self.LightColour(self._bottomStartColour, 40)
        self._topEndColour = self.LightColour(self._bottomStartColour, 25)
        self._pressedTopColour = self.LightColour(self._bottomStartColour, 20)
        self._pressedBottomColour = wx.Colour(*rgba)
        self.SetForegroundColour(foregroundcolour)

    def LightColour(self, colour: wx.Colour, percent: int) -> wx.Colour:
        """Return light contrast of `colour`.

        The colour returned is from the scale of `colour` ==> white.

        Args:
            colour (wx.Colour): the input colour to be brightened;
            percent (int): determines how light the colour will be.
                `percent` = 100 returns white, `percent` = 0 returns `colour`.

        Returns:
            wx.Colour: The lightened colour.
        """
        end_colour = wx.WHITE
        rd = end_colour.Red() - colour.Red()
        gd = end_colour.Green() - colour.Green()
        bd = end_colour.Blue() - colour.Blue()
        high = 100

        # We take the percent way of the colour from colour -. white
        i = percent
        r = colour.Red() + ((i * rd * 100) / high) / 100
        g = colour.Green() + ((i * gd * 100) / high) / 100
        b = colour.Blue() + ((i * bd * 100) / high) / 100
        a = colour.Alpha()

        return wx.Colour(int(r), int(g), int(b), int(a))

    def OnSize(self, event: wx.SizeEvent) -> None:
        """Handle the ``wx.EVT_SIZE`` event for :class:`GradientButton`.

        Args:
            event (wx.SizeEvent): a :class:`wx.SizeEvent` event to be processed.
        """
        event.Skip()
        self.Refresh()

    def OnLeftDown(self, event: wx.MouseEvent) -> None:
        """Handle the ``wx.EVT_LEFT_DOWN`` event for :class:`GradientButton`.

        Args:
            event (wx.MouseEvent): a :class:`MouseEvent` event to be processed.
        """
        if not self.IsEnabled():
            return

        self._mouseAction = CLICK
        self.CaptureMouse()
        self.Refresh()
        event.Skip()

    def OnLeftUp(self, event: wx.MouseEvent) -> None:
        """Handle the ``wx.EVT_LEFT_UP`` event for :class:`GradientButton`.

        Args:
            event (wx.MouseEvent): a :class:`MouseEvent` event to be processed.
        """
        if not self.IsEnabled() or not self.HasCapture():
            return

        pos = event.GetPosition()
        rect = self.GetClientRect()

        if self.HasCapture():
            self.ReleaseMouse()

        if rect.Contains(pos):
            self._mouseAction = HOVER
            self.Notify()
        else:
            self._mouseAction = None

        self.Refresh()
        event.Skip()

    def OnMouseEnter(self, event: wx.MouseEvent) -> None:
        """Handle the ``wx.EVT_ENTER_WINDOW`` event for :class:`GradientButton`.

        Args:
            event (wx.MouseEvent): a :class:`MouseEvent` event to be processed.
        """
        if not self.IsEnabled():
            return

        self._mouseAction = HOVER
        self.Refresh()
        event.Skip()

    def OnMouseLeave(self, event: wx.MouseEvent) -> None:
        """Handle the ``wx.EVT_LEAVE_WINDOW`` event for :class:`GradientButton`.

        Args:
            event (wx.MouseEvent): a :class:`MouseEvent` event to be processed.
        """
        self._mouseAction = None
        self.Refresh()
        event.Skip()

    def OnGainFocus(self, event: wx.FocusEvent) -> None:
        """Handle the ``wx.EVT_SET_FOCUS`` event for :class:`GradientButton`.

        Args:
            event (wx.FocusEvent): a :class:`FocusEvent` event to be processed.
        """
        self._hasFocus = True
        self.Refresh()
        self.Update()

    def OnLoseFocus(self, event: wx.FocusEvent) -> None:
        """Handle the ``wx.EVT_KILL_FOCUS`` event for :class:`GradientButton`.

        Args:
            event (wx.FocusEvent): a :class:`FocusEvent` event to be processed.
        """
        self._hasFocus = False
        self.Refresh()
        self.Update()

    def OnKeyDown(self, event: wx.KeyEvent) -> None:
        """Handle the ``wx.EVT_KEY_DOWN`` event for :class:`GradientButton`.

        Args:
            event (wx.KeyEvent): a :class:`KeyEvent` event to be processed.
        """
        if self._hasFocus and event.GetKeyCode() == ord(" "):
            self._mouseAction = HOVER
            self.Refresh()
        event.Skip()

    def OnKeyUp(self, event: wx.KeyEvent) -> None:
        """Handle the ``wx.EVT_KEY_UP`` event for :class:`GradientButton`.

        Args:
            event (wx.KeyEvent): a :class:`KeyEvent` event to be processed.
        """
        if self._hasFocus and event.GetKeyCode() == ord(" "):
            self._mouseAction = HOVER
            self.Notify()
            self.Refresh()
        event.Skip()

    def OnPaint(self, event: wx.PaintEvent) -> None:
        """Handle the ``wx.EVT_PAINT`` event for :class:`GradientButton`.

        Args:
            event (wx.PaintEvent): a :class:`PaintEvent` event to be processed.
        """
        dc = wx.BufferedPaintDC(self)
        gc = wx.GraphicsContext.Create(dc)
        dc.SetBackground(wx.Brush(self.GetParent().GetBackgroundColour()))
        dc.Clear()

        clientRect = self.GetClientRect()
        gradientRect = wx.Rect(*clientRect)
        capture = wx.Window.GetCapture()

        x, y, width, height = clientRect

        gradientRect.SetHeight(
            gradientRect.GetHeight() // 2 + ((capture == self and [1] or [0])[0])
        )
        if capture != self:
            if self._mouseAction == HOVER:
                topStart = self.LightColour(self._topStartColour, 10)
                topEnd = self.LightColour(self._topEndColour, 10)
            else:
                topStart, topEnd = self._topStartColour, self._topEndColour

            rc1 = wx.Rect(x, y, width, height // 2)
            path1 = self.GetPath(gc, rc1, 8)
            br1 = gc.CreateLinearGradientBrush(
                x, y, x, y + height / 2, topStart, topEnd
            )
            gc.SetBrush(br1)
            gc.FillPath(path1)  # draw main

            path4 = gc.CreatePath()
            path4.AddRectangle(x, y + height / 2 - 8, width, 8)
            path4.CloseSubpath()
            gc.SetBrush(br1)
            gc.FillPath(path4)

        else:
            rc1 = wx.Rect(x, y, width, height)
            path1 = self.GetPath(gc, rc1, 8)
            gc.SetPen(wx.Pen(self._pressedTopColour))
            gc.SetBrush(wx.Brush(self._pressedTopColour))
            gc.FillPath(path1)

        gradientRect.Offset((0, gradientRect.GetHeight()))

        if capture != self:
            if self._mouseAction == HOVER:
                bottomStart = self.LightColour(self._bottomStartColour, 10)
                bottomEnd = self.LightColour(self._bottomEndColour, 10)
            else:
                bottomStart, bottomEnd = self._bottomStartColour, self._bottomEndColour

            rc3 = wx.Rect(x, y + height // 2, width, height // 2)
            path3 = self.GetPath(gc, rc3, 8)
            br3 = gc.CreateLinearGradientBrush(
                x, y + height / 2, x, y + height, bottomStart, bottomEnd
            )
            gc.SetBrush(br3)
            gc.FillPath(path3)  # draw main

            path4 = gc.CreatePath()
            path4.AddRectangle(x, y + height / 2, width, 8)
            path4.CloseSubpath()
            gc.SetBrush(br3)
            gc.FillPath(path4)

            shadowOffset = 0
        else:
            rc2 = wx.Rect(
                x + 1, gradientRect.height // 2, gradientRect.width, gradientRect.height
            )
            path2 = self.GetPath(gc, rc2, 8)
            gc.SetPen(wx.Pen(self._pressedBottomColour))
            gc.SetBrush(wx.Brush(self._pressedBottomColour))
            gc.FillPath(path2)
            shadowOffset = 1

        # Create a ClientDC to get the text extent
        client_dc = wx.ClientDC(self)
        font = self.GetFont()
        client_dc.SetFont(font)
        label = self.GetLabel()
        tw, th = client_dc.GetTextExtent(label)

        if self._bitmap:
            bw = self._bitmap.GetWidth()
            bh = self._bitmap.GetHeight()
        else:
            bw = bh = 0

        # Set default values for pos_x and pos_y
        pos_x: float = 0.0
        pos_y: float = (height - th) / 2 + shadowOffset

        if self._alignment == wx.CENTER:
            # adjust for bitmap and text to centre
            pos_x = (width - bw - tw) / 2 + shadowOffset
            pos_y = (height - bh) / 2 + shadowOffset
            if self._bitmap:
                # draw bitmap if available
                gc.DrawBitmap(self._bitmap, pos_x, pos_y, bw, bh)
                pos_x += bw + 2  # extra spacing from bitmap
        elif self._alignment == wx.LEFT:
            pos_x = 3  # adjust for bitmap and text to left
            pos_y = (height - bh) / 2 + shadowOffset
            if self._bitmap:
                gc.DrawBitmap(
                    self._bitmap, pos_x, pos_y, bw, bh
                )  # draw bitmap if available
                pos_x += bw + 3  # extra spacing from bitmap

        gc.DrawText(label, pos_x + shadowOffset, pos_y)

    def GetPath(self, gc: wx.GraphicsContext, rc: wx.Rect, r: int) -> wx.GraphicsPath:
        """Return a rounded :class:`GraphicsPath` rectangle.

        Args:
            gc (wx.GraphicsContext): an instance of :class:`GraphicsContext`;
            rc (wx.Rect): a client rectangle;
            r (int): the radius of the rounded part of the rectangle.

        Returns:
            wx.GraphicsPath: A rounded rectangle path.
        """
        x, y, w, h = rc
        path = gc.CreatePath()
        path.AddRoundedRectangle(x, y, w, h, r)
        path.CloseSubpath()
        return path

    def SetInitialSize(self, size: Optional[wx.Size] = None) -> None:
        """
        Given the current font and bezel width settings, calculate and set a good size.

        Args:
            size (Optional[wx.Size]): an instance of :class:`wx.Size`.
        """
        if size is None:
            size = wx.DefaultSize
        wx.Control.SetInitialSize(self, size)

    SetBestSize = SetInitialSize

    def AcceptsFocus(self) -> bool:
        """Return True if this window can be given focus by mouse click.

        Note:
            Overridden from :class:`wx.Control`.

        Returns:
            bool: True if the window can be given focus by mouse click, False otherwise.
        """
        return self.IsShown() and self.IsEnabled()

    def GetDefaultAttributes(self) -> wx.VisualAttributes:
        """Overridden base class virtual.

        By default we should use the same font/colour attributes as the native
        :class:`Button`.

        Returns:
            wx.VisualAttributes: The default visual attributes for the button.
        """
        return wx.Button.GetClassDefaultAttributes()

    def ShouldInheritColours(self) -> bool:
        """Overridden base class virtual.

        Buttons usually don't inherit the parent's colours.

        Note:
            Overridden from :class:`wx.Control`.

        Returns:
            bool: True if the button should inherit the parent's colours,
                False otherwise.
        """
        return False

    def Enable(self, enable: bool = True) -> None:
        """Enable/disable the button.

        Args:
            enable (bool): ``True`` to enable the button, ``False`` to disable it.

        Note:
            Overridden from :class:`wx.Control`.
        """
        wx.Control.Enable(self, enable)
        self.Refresh()

    def SetTopStartColour(self, colour: wx.Colour) -> None:
        """Set the top start colour for the gradient shading.

        Args:
            colour (wx.Colour): a valid :class:`wx.Colour` object.
        """
        self._topStartColour = colour
        self.Refresh()

    def GetTopStartColour(self) -> wx.Colour:
        """Return the top start colour for the gradient shading.

        Returns:
            wx.Colour: The top start colour for the gradient shading.
        """
        return self._topStartColour

    def SetTopEndColour(self, colour: wx.Colour) -> None:
        """Set the top end colour for the gradient shading.

        Args:
            colour (wx.Colour): a valid :class:`wx.Colour` object.
        """
        self._topEndColour = colour
        self.Refresh()

    def GetTopEndColour(self) -> wx.Colour:
        """Return the top end colour for the gradient shading.

        Returns:
            wx.Colour: The top end colour of the gradient.
        """
        return self._topEndColour

    def SetBottomStartColour(self, colour: wx.Colour) -> None:
        """Set the top bottom colour for the gradient shading.

        Args:
            colour (wx.Colour): a valid :class:`wx.Colour` object.
        """
        self._bottomStartColour = colour
        self.Refresh()

    def GetBottomStartColour(self) -> wx.Colour:
        """Return the bottom start colour for the gradient shading.

        Returns:
            wx.Colour: The bottom start colour for the gradient shading.
        """
        return self._bottomStartColour

    def SetBottomEndColour(self, colour: wx.Colour) -> None:
        """Set the bottom end colour for the gradient shading.

        Args:
            colour (wx.Colour): a valid :class:`wx.Colour` object.
        """
        self._bottomEndColour = colour
        self.Refresh()

    def GetBottomEndColour(self) -> wx.Colour:
        """Return the bottom end colour for the gradient shading.

        Returns:
            wx.Colour: The bottom end colour of the gradient.
        """
        return self._bottomEndColour

    def SetPressedTopColour(self, colour: wx.Colour) -> None:
        """Set the pressed top start colour for the gradient shading.

        Args:
            colour (wx.Colour): a valid :class:`wx.Colour` object.
        """
        self._pressedTopColour = colour
        self.Refresh()

    def GetPressedTopColour(self) -> wx.Colour:
        """Return the pressed top start colour for the gradient shading.

        Returns:
            wx.Colour: The pressed top start colour.
        """
        return self._pressedTopColour

    def SetPressedBottomColour(self, colour: wx.Colour) -> None:
        """Set the pressed bottom start colour for the gradient shading.

        Args:
            colour (wx.Colour): a valid :class:`wx.Colour` object.
        """
        self._pressedBottomColour: wx.Colour = colour
        self.Refresh()

    def GetPressedBottomColour(self) -> wx.Colour:
        """Return the pressed bottom start colour for the gradient shading.

        Returns:
            wx.Colour: The pressed bottom start colour for the gradient shading.
        """
        return self._pressedBottomColour

    def SetForegroundColour(self, colour: wx.Colour) -> None:
        """Set the :class:`GradientButton` foreground (text) colour.

        Args:
            colour (wx.Colour): a valid :class:`wx.Colour` object.

        Note:
            Overridden from :class:`wx.Control`.
        """
        wx.Control.SetForegroundColour(self, colour)
        self.Refresh()

    def DoGetBestSize(self) -> wx.Size:
        """Overridden base class virtual.

        Determines the best size of the button based on the label and bezel size.

        Note:
            Overridden from :class:`wx.Control`.

        Returns:
            :class:`wx.Size`: The best size for the button.
        """
        label = self.GetLabel()
        if not label:
            return wx.Size(112, 48)

        dc = wx.ClientDC(self)
        dc.SetFont(self.GetFont())
        retWidth, retHeight = dc.GetTextExtent(label)

        bmpWidth = 0
        bmpHeight = 0
        constant = 15
        if self._bitmap:
            bmpWidth, bmpHeight = self._bitmap.GetWidth() + 10, self._bitmap.GetHeight()
            retWidth += bmpWidth
            retHeight = max(bmpHeight, retHeight)
            constant = 15

        return wx.Size(retWidth + constant, retHeight + constant)

    def SetDefault(self) -> None:
        """Set the default button."""
        tlw = wx.GetTopLevelParent(self)
        if isinstance(tlw, (wx.Dialog, wx.Frame)) and hasattr(tlw, "SetDefaultItem"):
            tlw.SetDefaultItem(self)
        else:
            # Fallback: Set the button as the default in a different way if possible
            if hasattr(self, "SetDefault"):
                self.SetDefault()

    def Notify(self) -> None:
        """Actually send a ``wx.EVT_BUTTON`` event to the listener (if any)."""
        evt = GradientButtonEvent(wx.wxEVT_COMMAND_BUTTON_CLICKED, self.GetId())
        evt.SetButtonObj(self)
        evt.SetEventObject(self)
        self.GetEventHandler().ProcessEvent(evt)



# -*- coding: UTF-8 -*-
"""
Interactive display calibration UI

"""

from time import sleep, strftime
import os
import re
import sys

from DisplayCAL.wxaddons import wx

from DisplayCAL.config import (
    getbitmap,
    getcfg,
    get_icon_bundle,
    get_display_number,
    get_display_rects,
    get_verified_path,
    setcfg,
)
from DisplayCAL.log import get_file_logger
from DisplayCAL.meta import name as appname, version as appversion
from DisplayCAL.util_os import launch_file, waccess
from DisplayCAL.wxaddons import CustomEvent
from DisplayCAL.wxMeasureFrame import MeasureFrame
from DisplayCAL.wxwindows import (
    BaseApp,
    BaseFrame,
    FlatShadedButton,
    numpad_keycodes,
    nav_keycodes,
    processing_keycodes,
    wx_Panel,
)
from DisplayCAL import colormath
from DisplayCAL import config
from DisplayCAL import localization as lang
from DisplayCAL import report

BGCOLOUR = wx.Colour(0x33, 0x33, 0x33)


class FlatShadedNumberedButton(FlatShadedButton):
    def __init__(
        self,
        parent,
        id=wx.ID_ANY,
        bitmap=None,
        label="",
        pos=wx.DefaultPosition,
        size=wx.DefaultSize,
        style=wx.NO_BORDER,
        validator=wx.DefaultValidator,
        name="gradientbutton",
        bgcolour=None,
        fgcolour=None,
        index=0,
    ):
        FlatShadedButton.__init__(
            self,
            parent,
            id,
            bitmap,
            label,
            pos,
            size,
            style,
            validator,
            name,
            bgcolour,
            fgcolour,
        )
        self.index = index

    def OnGainFocus(self, event):
        self.TopLevelParent.index = self.index
        FlatShadedButton.OnGainFocus(self, event)


class DisplayUniformityFrame(BaseFrame):
    def __init__(
        self,
        parent=None,
        handler=None,
        keyhandler=None,
        start_timer=True,
        rows=None,
        cols=None,
    ):
        if not rows:
            rows = getcfg("uniformity.rows")
        if not cols:
            cols = getcfg("uniformity.cols")
        BaseFrame.__init__(
            self,
            parent,
            wx.ID_ANY,
            lang.getstr("report.uniformity"),
            style=wx.DEFAULT_FRAME_STYLE | wx.TAB_TRAVERSAL,
            name="displayuniformityframe",
        )
        self.SetIcons(get_icon_bundle([256, 48, 32, 16], appname))
        self.SetBackgroundColour(BGCOLOUR)
        self.sizer = wx.GridSizer(rows, cols, 0, 0)
        self.SetSizer(self.sizer)

        self.rows = rows
        self.cols = cols
        self.colors = (
            wx.WHITE,
            wx.Colour(192, 192, 192),
            wx.Colour(128, 128, 128),
            wx.Colour(64, 64, 64),
        )
        self.labels = {}
        self.panels = []
        self.buttons = []
        for index in range(rows * cols):
            panel = wx_Panel(self, style=wx.BORDER_SIMPLE)
            panel.SetBackgroundColour(BGCOLOUR)
            sizer = wx.BoxSizer(wx.VERTICAL)
            panel.SetSizer(sizer)
            self.panels.append(panel)
            button = FlatShadedNumberedButton(
                panel,
                label=lang.getstr("measure"),
                bitmap=getbitmap("theme/icons/10x10/record"),
                index=index,
            )
            button.Bind(wx.EVT_BUTTON, self.measure)
            self.buttons.append(button)
            label = wx.StaticText(panel)
            label.SetForegroundColour(wx.WHITE)
            self.labels[index] = label
            sizer.Add(label, 1, wx.ALIGN_CENTER)
            sizer.Add(
                button,
                0,
                wx.ALIGN_CENTER | wx.BOTTOM,
                # | wx.LEFT | wx.RIGHT,
                border=8,
            )
            self.sizer.Add(panel, 1, wx.EXPAND)
        self.disable_buttons()

        self.keyhandler = keyhandler
        self.id_to_keycode = {}
        if sys.platform == "darwin":
            # Use an accelerator table for tab, space, 0-9, A-Z, numpad,
            # navigation keys and processing keys
            keycodes = [wx.WXK_TAB, wx.WXK_SPACE]
            keycodes.extend(list(range(ord("0"), ord("9"))))
            keycodes.extend(list(range(ord("A"), ord("Z"))))
            keycodes.extend(numpad_keycodes)
            keycodes.extend(nav_keycodes)
            keycodes.extend(processing_keycodes)
            for keycode in keycodes:
                self.id_to_keycode[wx.Window.NewControlId()] = keycode
            accels = []
            for id in self.id_to_keycode:
                keycode = self.id_to_keycode[id]
                self.Bind(wx.EVT_MENU, self.key_handler, id=id)
                accels.append((wx.ACCEL_NORMAL, keycode, id))
                if keycode == wx.WXK_TAB:
                    accels.append((wx.ACCEL_SHIFT, keycode, id))
            self.SetAcceleratorTable(wx.AcceleratorTable(accels))
        else:
            self.Bind(wx.EVT_CHAR_HOOK, self.key_handler)

        # Event handlers
        self.Bind(wx.EVT_CLOSE, self.OnClose, self)
        self.Bind(wx.EVT_MOVE, self.OnMove, self)
        self.timer = wx.Timer(self)
        if handler:
            self.Bind(wx.EVT_TIMER, handler, self.timer)
        self.Bind(wx.EVT_WINDOW_DESTROY, self.OnDestroy, self)

        # Final initialization steps
        self.logger = get_file_logger("uniformity")
        self._setup()

        self.Show()

        if start_timer:
            self.start_timer()

    def EndModal(self, returncode=wx.ID_OK):
        return returncode

    def MakeModal(self, makemodal=False):
        pass

    def OnClose(self, event):
        if not self.timer.IsRunning():
            self.Destroy()
        else:
            self.keepGoing = False

    def OnDestroy(self, event):
        self.stop_timer()
        del self.timer
        if not hasattr(wx.Window, "UnreserveControlId"):
            return 0
        for id in self.id_to_keycode.keys():
            if id >= 0:
                continue
            try:
                wx.Window.UnreserveControlId(id)
            except wx.wxAssertionError as exception:
                print(exception)
        return 0

    def OnMove(self, event):
        pass

    def Pulse(self, msg=""):
        return self.keepGoing, False

    def Resume(self):
        self.keepGoing = True

    def Show(self, show=True):
        if show:
            display_no = getcfg("display.number") - 1
            if display_no < 0 or display_no > wx.Display.GetCount() - 1:
                display_no = 0
            else:
                display_no = get_display_number(display_no)
            x, y, w, h = wx.Display(display_no).ClientArea
            # Place frame on correct display
            self.SetPosition((x, y))
            self.SetSize((w, h))
            self.disable_buttons()
            wx.CallAfter(self.Maximize)
        wx.Frame.Show(self, show)
        self.panels[0].SetFocus()

    def UpdateProgress(self, value, msg=""):
        return self.Pulse(msg)

    def UpdatePulse(self, msg=""):
        return self.Pulse(msg)

    def disable_buttons(self):
        self.enable_buttons(False)

    def enable_buttons(self, enable=True):
        for button in self.buttons:
            button.Enable(enable)

    def flush(self):
        pass

    get_display = MeasureFrame.__dict__["get_display"]

    def has_worker_subprocess(self):
        return bool(
            getattr(self, "worker", None) and getattr(self.worker, "subprocess", None)
        )

    def hide_cursor(self):
        cursor_id = wx.CURSOR_BLANK
        cursor = wx.StockCursor(cursor_id)
        self.SetCursor(cursor)
        for panel in self.panels:
            panel.SetCursor(cursor)
        for label in list(self.labels.values()):
            label.SetCursor(cursor)
        for button in self.buttons:
            button.SetCursor(cursor)

    def isatty(self):
        return True

    def key_handler(self, event):
        keycode = None
        if event.GetEventType() in (
            wx.EVT_CHAR.typeId,
            wx.EVT_CHAR_HOOK.typeId,
            wx.EVT_KEY_DOWN.typeId,
        ):
            keycode = event.GetKeyCode()
        elif event.GetEventType() == wx.EVT_MENU.typeId:
            keycode = self.id_to_keycode.get(event.GetId())
        if keycode == wx.WXK_TAB:
            self.global_navigate() or event.Skip()
        elif keycode >= 0:
            if self.has_worker_subprocess() and keycode < 256:
                if keycode == wx.WXK_ESCAPE or chr(keycode) == "Q":
                    # ESC or Q
                    self.worker.abort_subprocess()
                elif (
                    self.index > -1
                    and not self.is_measuring
                    and (
                        not isinstance(self.FindFocus(), wx.Control)
                        or keycode != wx.WXK_SPACE
                    )
                ):
                    # Any other key
                    self.measure(
                        CustomEvent(wx.EVT_BUTTON.typeId, self.buttons[self.index])
                    )
                else:
                    event.Skip()
            else:
                event.Skip()
        else:
            event.Skip()

    def measure(self, event=None):
        if event:
            self.index = event.GetEventObject().index
            print("%s: Uniformity grid index %i" % (appname, self.index))
            self.is_measuring = True
            self.results[self.index] = []
            self.labels[self.index].SetLabel("")
            self.hide_cursor()
            self.disable_buttons()
            self.buttons[self.index].Hide()
        self.panels[self.index].SetBackgroundColour(
            self.colors[len(self.results[self.index])]
        )
        self.panels[self.index].Refresh()
        self.panels[self.index].Update()
        print(
            "%s: About to measure uniformity grid index %i @%i%%"
            % (
                appname,
                self.index,
                self.colors[len(self.results[self.index])].red / 2.55,
            )
        )
        # Use a delay to allow for TFT lag
        wx.CallLater(200, self.safe_send, " ")

    def parse_txt(self, txt):
        if not txt:
            return
        self.logger.info("%r" % txt)
        if "Setting up the instrument" in txt:
            self.Pulse(lang.getstr("instrument.initializing"))
        if "Spot read failed" in txt:
            self.last_error = txt
        if "Result is XYZ:" in txt:
            # Result is XYZ: d.dddddd d.dddddd d.dddddd, D50 Lab: d.dddddd d.dddddd d.dddddd
            # 							CCT = ddddK (Delta E d.dddddd)
            # Closest Planckian temperature = ddddK (Delta E d.dddddd)
            # Closest Daylight temperature  = ddddK (Delta E d.dddddd)
            XYZ = re.search(r"XYZ:\s+(\d+\.\d+)\s+(\d+\.\d+)\s+(\d+\.\d+)", txt)
            self.results[self.index].append(
                {"XYZ": [float(value) for value in XYZ.groups()]}
            )
            self.last_error = None
        loci = {"t": "Daylight", "T": "Planckian"}
        for locus in list(loci.values()):
            if locus in txt:
                CT = re.search(
                    r"Closest\s+%s\s+temperature\s+=\s+(\d+)K" % locus, txt, re.I
                )
                self.results[self.index][-1]["C%sT" % locus[0]] = int(CT.groups()[0])
        if "key to take a reading" in txt and not self.last_error:
            print("%s: Got 'key to take a reading'" % appname)
            if not self.is_measuring:
                self.enable_buttons()
                return
            if len(self.results[self.index]) < len(self.colors):
                # Take readings at 5 different brightness levels per swatch
                print("%s: About to take next reading" % appname)
                self.measure()
            else:
                self.is_measuring = False
                self.show_cursor()
                self.enable_buttons()
                self.buttons[self.index].Show()
                self.buttons[self.index].SetFocus()
                self.buttons[self.index].SetBitmap(
                    getbitmap("theme/icons/16x16/checkmark")
                )
                self.panels[self.index].SetBackgroundColour(BGCOLOUR)
                self.panels[self.index].Refresh()
                self.panels[self.index].Update()
                if len(self.results) == self.rows * self.cols:
                    # All swatches have been measured, show results
                    # Let the user choose a location for the results html
                    display_no, geometry, client_area = self.get_display()
                    # Translate from wx display index to Argyll display index
                    geometry = "%i, %i, %ix%i" % tuple(geometry)
                    for i, display in enumerate(getcfg("displays")):
                        if display.find("@ " + geometry) > -1:
                            print("Found display %s at index %i" % (display, i))
                            break
                    display = display.replace(" [PRIMARY]", "")
                    defaultFile = "Uniformity Check %s — %s — %s" % (
                        appversion,
                        re.sub(r"[\\/:*?\"<>|]+", "_", display),
                        strftime("%Y-%m-%d %H-%M.html"),
                    )
                    defaultDir = get_verified_path(
                        None, os.path.join(getcfg("profile.save_path"), defaultFile)
                    )[0]
                    dlg = wx.FileDialog(
                        self,
                        lang.getstr("save_as"),
                        defaultDir,
                        defaultFile,
                        wildcard=lang.getstr("filetype.html") + "|*.html;*.htm",
                        style=wx.SAVE | wx.FD_OVERWRITE_PROMPT,
                    )
                    dlg.Center(wx.BOTH)
                    result = dlg.ShowModal()
                    if result == wx.ID_OK:
                        path = dlg.GetPath()
                        if not waccess(path, os.W_OK):
                            from DisplayCAL.worker import show_result_dialog

                            show_result_dialog(
                                Error(lang.getstr("error.access_denied.write", path)),
                                self,
                            )
                            return
                        save_path = os.path.splitext(path)[0] + ".html"
                        setcfg("last_filedialog_path", save_path)
                    dlg.Destroy()
                    if result != wx.ID_OK:
                        return
                    locus = loci.get(getcfg("whitepoint.colortemp.locus"))
                    try:
                        report.create(
                            save_path,
                            {
                                "${REPORT_VERSION}": appversion,
                                "${DISPLAY}": display,
                                "${DATETIME}": strftime("%Y-%m-%d %H:%M:%S"),
                                "${ROWS}": str(self.rows),
                                "${COLS}": str(self.cols),
                                "${RESULTS}": str(self.results),
                                "${LOCUS}": locus,
                            },
                            getcfg("report.pack_js"),
                            "uniformity",
                        )
                    except (IOError, OSError) as exception:
                        from DisplayCAL.worker import show_result_dialog

                        show_result_dialog(exception, self)
                    else:
                        launch_file(save_path)
                if getcfg("uniformity.measure.continuous"):
                    self.measure(event=Event(self.buttons[self.index]))

    def reset(self):
        self._setup()
        for panel in self.panels:
            panel.SetBackgroundColour(BGCOLOUR)
        for button in self.buttons:
            button.SetBitmap(getbitmap("theme/icons/10x10/record"))
            button.Show()
        for index in self.labels:
            self.labels[index].SetLabel("")
            self.labels[index].GetContainingSizer().Layout()
        self.show_cursor()

    def _setup(self):
        self.logger.info("-" * 80)
        self.index = 0
        self.is_measuring = False
        self.keepGoing = True
        self.last_error = None
        self.results = {}
        self.display_rects = get_display_rects()

    def safe_send(self, bytes):
        if self.has_worker_subprocess() and not self.worker.subprocess_abort:
            if not self.worker.instrument_on_screen:
                if not getattr(self, "wait_for_instrument_on_screen", False):
                    self.wait_for_instrument_on_screen = True
                    print("%s: Waiting for instrument to be placed on screen" % appname)
                wx.CallLater(200, self.safe_send, bytes)
            else:
                self.wait_for_instrument_on_screen = False
                self.worker.safe_send(bytes)

    def show_cursor(self):
        cursor = wx.StockCursor(wx.CURSOR_ARROW)
        self.SetCursor(cursor)
        for panel in self.panels:
            panel.SetCursor(cursor)
        for label in list(self.labels.values()):
            label.SetCursor(cursor)
        for button in self.buttons:
            button.SetCursor(cursor)

    def start_timer(self, ms=50):
        self.timer.Start(ms)

    def stop_timer(self):
        self.timer.Stop()

    def write(self, txt):
        wx.CallAfter(self.parse_txt, txt)


class Event:
    def __init__(self, evtobj):
        self.evtobj = evtobj

    def GetEventObject(self):
        return self.evtobj


if __name__ == "__main__":
    from _thread import start_new_thread
    from time import sleep

    class Subprocess:
        def send(self, bytes_):
            start_new_thread(test, (bytes_,))

    class Worker:
        def __init__(self):
            self.subprocess = Subprocess()
            self.subprocess_abort = False

        def abort_subprocess(self):
            self.subprocess.send("Q")

        def safe_send(self, bytes_):
            self.subprocess.send(bytes_)
            return True

    config.initcfg()
    lang.init()
    lang.update_defaults()
    app = BaseApp(0)
    app.TopWindow = DisplayUniformityFrame(start_timer=False, rows=3, cols=3)
    app.TopWindow.worker = Worker()
    app.TopWindow.Show()
    i = 0

    def test(bytes_=None):
        global i
        menu = r"""Place instrument on spot to be measured,
and hit [A-Z] to read white and setup FWA compensation (keyed to letter)
[a-z] to read and make FWA compensated reading from keyed reference
'r' to set reference, 's' to save spectrum,
'h' to toggle high res., 'k' to do a calibration
Hit ESC or Q to exit, any other key to take a reading:"""
        if not bytes_:
            txt = menu
        elif bytes_ == " ":
            txt = [
                [
                    """
 Result is XYZ: 115.629826 123.903717 122.761510, D50 Lab: 108.590836 -5.813746 -13.529075
                           CCT = 6104K (Delta E 7.848119)
 Closest Planckian temperature = 5835K (Delta E 6.927113)
 Closest Daylight temperature  = 5963K (Delta E 3.547392)

Place instrument on spot to be measured,
and hit [A-Z] to read white and setup FWA compensation (keyed to letter)
[a-z] to read and make FWA compensated reading from keyed reference
'r' to set reference, 's' to save spectrum,
'h' to toggle high res., 'k' to do a calibration
Hit ESC or Q to exit, any other key to take a reading:""",
                    """
 Result is XYZ: 65.336831 69.578641 68.180005, D50 Lab: 86.789788 -3.888434 -10.469442
                           CCT = 5983K (Delta E 6.816507)
 Closest Planckian temperature = 5757K (Delta E 5.996638)
 Closest Daylight temperature  = 5883K (Delta E 2.598118)

Place instrument on spot to be measured,
and hit [A-Z] to read white and setup FWA compensation (keyed to letter)
[a-z] to read and make FWA compensated reading from keyed reference
'r' to set reference, 's' to save spectrum,
'h' to toggle high res., 'k' to do a calibration
Hit ESC or Q to exit, any other key to take a reading:""",
                    """
 Result is XYZ: 26.944662 28.614568 28.107897, D50 Lab: 60.439948 -2.589848 -7.899247
                           CCT = 5969K (Delta E 6.279024)
 Closest Planckian temperature = 5760K (Delta E 5.519000)
 Closest Daylight temperature  = 5887K (Delta E 2.119333)

Place instrument on spot to be measured,
and hit [A-Z] to read white and setup FWA compensation (keyed to letter)
[a-z] to read and make FWA compensated reading from keyed reference
'r' to set reference, 's' to save spectrum,
'h' to toggle high res., 'k' to do a calibration
Hit ESC or Q to exit, any other key to take a reading:""",
                    """
 Result is XYZ: 6.153402 6.500147 6.625585, D50 Lab: 30.640770 -1.226804 -5.876967
                           CCT = 6123K (Delta E 4.946609)
 Closest Planckian temperature = 5943K (Delta E 4.353019)
 Closest Daylight temperature  = 6082K (Delta E 0.985734)

Place instrument on spot to be measured,
and hit [A-Z] to read white and setup FWA compensation (keyed to letter)
[a-z] to read and make FWA compensated reading from keyed reference
'r' to set reference, 's' to save spectrum,
'h' to toggle high res., 'k' to do a calibration
Hit ESC or Q to exit, any other key to take a reading:""",
                    """
 Result is XYZ: 0.104401 0.110705 0.109155, D50 Lab: 0.640770 -1.226804 -5.876967
                           CCT = 6123K (Delta E 4.946609)
 Closest Planckian temperature = 5943K (Delta E 4.353019)
 Closest Daylight temperature  = 6082K (Delta E 0.985734)

Place instrument on spot to be measured,
and hit [A-Z] to read white and setup FWA compensation (keyed to letter)
[a-z] to read and make FWA compensated reading from keyed reference
'r' to set reference, 's' to save spectrum,
'h' to toggle high res., 'k' to do a calibration
Hit ESC or Q to exit, any other key to take a reading:""",
                ],
                [
                    """
 Result is XYZ: 116.565941 124.165894 121.365684, D50 Lab: 108.678651 -4.762572 -12.508939
                           CCT = 5972K (Delta E 6.890329)
 Closest Planckian temperature = 5745K (Delta E 6.060831)
 Closest Daylight temperature  = 5871K (Delta E 2.660205)

Place instrument on spot to be measured,
and hit [A-Z] to read white and setup FWA compensation (keyed to letter)
[a-z] to read and make FWA compensated reading from keyed reference
'r' to set reference, 's' to save spectrum,
'h' to toggle high res., 'k' to do a calibration
Hit ESC or Q to exit, any other key to take a reading:""",
                    """
 Result is XYZ: 64.511790 68.522425 66.980369, D50 Lab: 86.267011 -3.491468 -10.263432
                           CCT = 5945K (Delta E 6.363056)
 Closest Planckian temperature = 5735K (Delta E 5.590753)
 Closest Daylight temperature  = 5862K (Delta E 2.186503)

Place instrument on spot to be measured,
and hit [A-Z] to read white and setup FWA compensation (keyed to letter)
[a-z] to read and make FWA compensated reading from keyed reference
'r' to set reference, 's' to save spectrum,
'h' to toggle high res., 'k' to do a calibration
Hit ESC or Q to exit, any other key to take a reading:""",
                    """
 Result is XYZ: 26.905684 28.417087 27.988741, D50 Lab: 60.263695 -1.987837 -8.005457
                           CCT = 5930K (Delta E 5.234243)
 Closest Planckian temperature = 5755K (Delta E 4.591707)
 Closest Daylight temperature  = 5884K (Delta E 1.187672)

Place instrument on spot to be measured,
and hit [A-Z] to read white and setup FWA compensation (keyed to letter)
[a-z] to read and make FWA compensated reading from keyed reference
'r' to set reference, 's' to save spectrum,
'h' to toggle high res., 'k' to do a calibration
Hit ESC or Q to exit, any other key to take a reading:""",
                    """
 Result is XYZ: 6.144071 6.471379 6.584408, D50 Lab: 30.571861 -1.030833 -5.816641
                           CCT = 6083K (Delta E 4.418192)
 Closest Planckian temperature = 5923K (Delta E 3.883022)
 Closest Daylight temperature  = 6062K (Delta E 0.510176)

Place instrument on spot to be measured,
and hit [A-Z] to read white and setup FWA compensation (keyed to letter)
[a-z] to read and make FWA compensated reading from keyed reference
'r' to set reference, 's' to save spectrum,
'h' to toggle high res., 'k' to do a calibration
Hit ESC or Q to exit, any other key to take a reading:""",
                    """
 Result is XYZ: 0.104401 0.110705 0.109155, D50 Lab: 0.571861 -1.030833 -5.816641
                           CCT = 6083K (Delta E 4.418192)
 Closest Planckian temperature = 5923K (Delta E 3.883022)
 Closest Daylight temperature  = 6062K (Delta E 0.510176)

Place instrument on spot to be measured,
and hit [A-Z] to read white and setup FWA compensation (keyed to letter)
[a-z] to read and make FWA compensated reading from keyed reference
'r' to set reference, 's' to save spectrum,
'h' to toggle high res., 'k' to do a calibration
Hit ESC or Q to exit, any other key to take a reading:""",
                ],
                [
                    """
 Result is XYZ: 116.611176 123.928350 121.363808, D50 Lab: 108.599092 -4.350754 -12.644938
                           CCT = 5960K (Delta E 6.444925)
 Closest Planckian temperature = 5747K (Delta E 5.664879)
 Closest Daylight temperature  = 5873K (Delta E 2.263144)

Place instrument on spot to be measured,
and hit [A-Z] to read white and setup FWA compensation (keyed to letter)
[a-z] to read and make FWA compensated reading from keyed reference
'r' to set reference, 's' to save spectrum,
'h' to toggle high res., 'k' to do a calibration
Hit ESC or Q to exit, any other key to take a reading:""",
                    """
 Result is XYZ: 64.672460 68.441938 67.178377, D50 Lab: 86.226954 -2.956057 -10.516177
                           CCT = 5931K (Delta E 5.640857)
 Closest Planckian temperature = 5744K (Delta E 4.950818)
 Closest Daylight temperature  = 5872K (Delta E 1.545901)

Place instrument on spot to be measured,
and hit [A-Z] to read white and setup FWA compensation (keyed to letter)
[a-z] to read and make FWA compensated reading from keyed reference
'r' to set reference, 's' to save spectrum,
'h' to toggle high res., 'k' to do a calibration
Hit ESC or Q to exit, any other key to take a reading:""",
                    """
 Result is XYZ: 26.708397 28.224354 27.854805, D50 Lab: 60.090889 -2.043543 -8.080532
                           CCT = 5946K (Delta E 5.317449)
 Closest Planckian temperature = 5768K (Delta E 4.666630)
 Closest Daylight temperature  = 5897K (Delta E 1.265350)

Place instrument on spot to be measured,
and hit [A-Z] to read white and setup FWA compensation (keyed to letter)
[a-z] to read and make FWA compensated reading from keyed reference
'r' to set reference, 's' to save spectrum,
'h' to toggle high res., 'k' to do a calibration
Hit ESC or Q to exit, any other key to take a reading:""",
                    """
 Result is XYZ: 6.154005 6.469558 6.599849, D50 Lab: 30.567493 -0.904424 -5.891430
                           CCT = 6079K (Delta E 4.041262)
 Closest Planckian temperature = 5932K (Delta E 3.549922)
 Closest Daylight temperature  = 6072K (Delta E 0.177697)

Place instrument on spot to be measured,
and hit [A-Z] to read white and setup FWA compensation (keyed to letter)
[a-z] to read and make FWA compensated reading from keyed reference
'r' to set reference, 's' to save spectrum,
'h' to toggle high res., 'k' to do a calibration
Hit ESC or Q to exit, any other key to take a reading:""",
                    """
 Result is XYZ: 0.104401 0.110705 0.109155, D50 Lab: 0.567493 -0.904424 -5.891430
                           CCT = 6079K (Delta E 4.041262)
 Closest Planckian temperature = 5932K (Delta E 3.549922)
 Closest Daylight temperature  = 6072K (Delta E 0.177697)

Place instrument on spot to be measured,
and hit [A-Z] to read white and setup FWA compensation (keyed to letter)
[a-z] to read and make FWA compensated reading from keyed reference
'r' to set reference, 's' to save spectrum,
'h' to toggle high res., 'k' to do a calibration
Hit ESC or Q to exit, any other key to take a reading:""",
                ],
                [
                    """
 Result is XYZ: 120.030166 127.667344 125.560879, D50 Lab: 109.839774 -4.542272 -13.098348
                           CCT = 5991K (Delta E 6.554213)
 Closest Planckian temperature = 5772K (Delta E 5.765044)
 Closest Daylight temperature  = 5899K (Delta E 2.368586)

Place instrument on spot to be measured,
and hit [A-Z] to read white and setup FWA compensation (keyed to letter)
[a-z] to read and make FWA compensated reading from keyed reference
'r' to set reference, 's' to save spectrum,
'h' to toggle high res., 'k' to do a calibration
Hit ESC or Q to exit, any other key to take a reading:""",
                    """
 Result is XYZ: 66.590402 70.542611 69.377397, D50 Lab: 87.262309 -3.134252 -10.747149
                           CCT = 5951K (Delta E 5.807812)
 Closest Planckian temperature = 5758K (Delta E 5.100360)
 Closest Daylight temperature  = 5886K (Delta E 1.698719)

Place instrument on spot to be measured,
and hit [A-Z] to read white and setup FWA compensation (keyed to letter)
[a-z] to read and make FWA compensated reading from keyed reference
'r' to set reference, 's' to save spectrum,
'h' to toggle high res., 'k' to do a calibration
Hit ESC or Q to exit, any other key to take a reading:""",
                    """
 Result is XYZ: 27.489125 29.158690 28.738045, D50 Lab: 60.921426 -2.478038 -8.105322
                           CCT = 5976K (Delta E 6.028851)
 Closest Planckian temperature = 5773K (Delta E 5.298263)
 Closest Daylight temperature  = 5902K (Delta E 1.900430)

Place instrument on spot to be measured,
and hit [A-Z] to read white and setup FWA compensation (keyed to letter)
[a-z] to read and make FWA compensated reading from keyed reference
'r' to set reference, 's' to save spectrum,
'h' to toggle high res., 'k' to do a calibration
Hit ESC or Q to exit, any other key to take a reading:""",
                    """
 Result is XYZ: 6.326874 6.649709 6.776715, D50 Lab: 30.995780 -0.896754 -5.916062
                           CCT = 6071K (Delta E 4.005433)
 Closest Planckian temperature = 5926K (Delta E 3.517820)
 Closest Daylight temperature  = 6065K (Delta E 0.144142)

Place instrument on spot to be measured,
and hit [A-Z] to read white and setup FWA compensation (keyed to letter)
[a-z] to read and make FWA compensated reading from keyed reference
'r' to set reference, 's' to save spectrum,
'h' to toggle high res., 'k' to do a calibration
Hit ESC or Q to exit, any other key to take a reading:""",
                    """
 Result is XYZ: 0.104401 0.110705 0.109155, D50 Lab: 0.995780 -0.896754 -5.916062
                           CCT = 6071K (Delta E 4.005433)
 Closest Planckian temperature = 5926K (Delta E 3.517820)
 Closest Daylight temperature  = 6065K (Delta E 0.144142)

Place instrument on spot to be measured,
and hit [A-Z] to read white and setup FWA compensation (keyed to letter)
[a-z] to read and make FWA compensated reading from keyed reference
'r' to set reference, 's' to save spectrum,
'h' to toggle high res., 'k' to do a calibration
Hit ESC or Q to exit, any other key to take a reading:""",
                ],
                [
                    """
 Result is XYZ: 121.643262 130.105649 128.560173, D50 Lab: 110.635861 -5.574898 -13.543244
                           CCT = 6071K (Delta E 7.533820)
 Closest Planckian temperature = 5815K (Delta E 6.643605)
 Closest Daylight temperature  = 5943K (Delta E 3.258868)

Place instrument on spot to be measured,
and hit [A-Z] to read white and setup FWA compensation (keyed to letter)
[a-z] to read and make FWA compensated reading from keyed reference
'r' to set reference, 's' to save spectrum,
'h' to toggle high res., 'k' to do a calibration
Hit ESC or Q to exit, any other key to take a reading:""",
                    """
 Result is XYZ: 67.775736 72.266319 71.178047, D50 Lab: 88.096621 -4.123469 -10.928024
                           CCT = 6023K (Delta E 6.995424)
 Closest Planckian temperature = 5788K (Delta E 6.159783)
 Closest Daylight temperature  = 5915K (Delta E 2.767919)

Place instrument on spot to be measured,
and hit [A-Z] to read white and setup FWA compensation (keyed to letter)
[a-z] to read and make FWA compensated reading from keyed reference
'r' to set reference, 's' to save spectrum,
'h' to toggle high res., 'k' to do a calibration
Hit ESC or Q to exit, any other key to take a reading:""",
                    """
 Result is XYZ: 28.131948 29.978900 29.595129, D50 Lab: 61.636012 -3.012753 -8.258625
                           CCT = 6030K (Delta E 6.867980)
 Closest Planckian temperature = 5798K (Delta E 6.047536)
 Closest Daylight temperature  = 5926K (Delta E 2.657241)

Place instrument on spot to be measured,
and hit [A-Z] to read white and setup FWA compensation (keyed to letter)
[a-z] to read and make FWA compensated reading from keyed reference
'r' to set reference, 's' to save spectrum,
'h' to toggle high res., 'k' to do a calibration
Hit ESC or Q to exit, any other key to take a reading:""",
                    """
 Result is XYZ: 6.473470 6.765282 6.888164, D50 Lab: 31.266484 -0.517860 -5.923364
                           CCT = 6007K (Delta E 2.947859)
 Closest Planckian temperature = 5902K (Delta E 2.582843)
 Closest Daylight temperature  = 6042K (Delta E 0.798814)

Place instrument on spot to be measured,
and hit [A-Z] to read white and setup FWA compensation (keyed to letter)
[a-z] to read and make FWA compensated reading from keyed reference
'r' to set reference, 's' to save spectrum,
'h' to toggle high res., 'k' to do a calibration
Hit ESC or Q to exit, any other key to take a reading:""",
                    """
 Result is XYZ: 0.104401 0.110705 0.109155, D50 Lab: 0.266484 -0.517860 -5.923364
                           CCT = 6007K (Delta E 2.947859)
 Closest Planckian temperature = 5902K (Delta E 2.582843)
 Closest Daylight temperature  = 6042K (Delta E 0.798814)

Place instrument on spot to be measured,
and hit [A-Z] to read white and setup FWA compensation (keyed to letter)
[a-z] to read and make FWA compensated reading from keyed reference
'r' to set reference, 's' to save spectrum,
'h' to toggle high res., 'k' to do a calibration
Hit ESC or Q to exit, any other key to take a reading:""",
                ],
                [
                    """
 Result is XYZ: 116.801943 124.624261 123.359911, D50 Lab: 108.831883 -5.063829 -13.483891
                           CCT = 6057K (Delta E 7.069302)
 Closest Planckian temperature = 5816K (Delta E 6.229078)
 Closest Daylight temperature  = 5944K (Delta E 2.843045)

Place instrument on spot to be measured,
and hit [A-Z] to read white and setup FWA compensation (keyed to letter)
[a-z] to read and make FWA compensated reading from keyed reference
'r' to set reference, 's' to save spectrum,
'h' to toggle high res., 'k' to do a calibration
Hit ESC or Q to exit, any other key to take a reading:""",
                    """
 Result is XYZ: 64.869350 68.842421 67.977252, D50 Lab: 86.425958 -3.370123 -10.910497
                           CCT = 5991K (Delta E 6.099488)
 Closest Planckian temperature = 5785K (Delta E 5.362276)
 Closest Daylight temperature  = 5914K (Delta E 1.966958)

Place instrument on spot to be measured,
and hit [A-Z] to read white and setup FWA compensation (keyed to letter)
[a-z] to read and make FWA compensated reading from keyed reference
'r' to set reference, 's' to save spectrum,
'h' to toggle high res., 'k' to do a calibration
Hit ESC or Q to exit, any other key to take a reading:""",
                    """
 Result is XYZ: 26.750888 28.302175 27.882982, D50 Lab: 60.160759 -2.171950 -8.007010
                           CCT = 5948K (Delta E 5.551435)
 Closest Planckian temperature = 5762K (Delta E 4.873477)
 Closest Daylight temperature  = 5891K (Delta E 1.471926)

Place instrument on spot to be measured,
and hit [A-Z] to read white and setup FWA compensation (keyed to letter)
[a-z] to read and make FWA compensated reading from keyed reference
'r' to set reference, 's' to save spectrum,
'h' to toggle high res., 'k' to do a calibration
Hit ESC or Q to exit, any other key to take a reading:""",
                    """
 Result is XYZ: 6.255018 6.563924 6.746680, D50 Lab: 30.792814 -0.788285 -6.137368
                           CCT = 6105K (Delta E 3.641727)
 Closest Planckian temperature = 5970K (Delta E 3.198805)
 Closest Daylight temperature  = 6113K (Delta E 0.167052)

Place instrument on spot to be measured,
and hit [A-Z] to read white and setup FWA compensation (keyed to letter)
[a-z] to read and make FWA compensated reading from keyed reference
'r' to set reference, 's' to save spectrum,
'h' to toggle high res., 'k' to do a calibration
Hit ESC or Q to exit, any other key to take a reading:""",
                    """
 Result is XYZ: 0.104401 0.110705 0.109155, D50 Lab: 0.792814 -0.788285 -6.137368
                           CCT = 6105K (Delta E 3.641727)
 Closest Planckian temperature = 5970K (Delta E 3.198805)
 Closest Daylight temperature  = 6113K (Delta E 0.167052)

Place instrument on spot to be measured,
and hit [A-Z] to read white and setup FWA compensation (keyed to letter)
[a-z] to read and make FWA compensated reading from keyed reference
'r' to set reference, 's' to save spectrum,
'h' to toggle high res., 'k' to do a calibration
Hit ESC or Q to exit, any other key to take a reading:""",
                ],
                [
                    """
 Result is XYZ: 115.081771 122.300809 119.963479, D50 Lab: 108.051237 -4.328489 -12.711256
                           CCT = 5969K (Delta E 6.425079)
 Closest Planckian temperature = 5755K (Delta E 5.648226)
 Closest Daylight temperature  = 5882K (Delta E 2.248055)

Place instrument on spot to be measured,
and hit [A-Z] to read white and setup FWA compensation (keyed to letter)
[a-z] to read and make FWA compensated reading from keyed reference
'r' to set reference, 's' to save spectrum,
'h' to toggle high res., 'k' to do a calibration
Hit ESC or Q to exit, any other key to take a reading:""",
                    """
 Result is XYZ: 62.334621 66.118418 64.979303, D50 Lab: 85.056784 -3.250929 -10.473103
                           CCT = 5960K (Delta E 6.051574)
 Closest Planckian temperature = 5758K (Delta E 5.316783)
 Closest Daylight temperature  = 5886K (Delta E 1.916058)

Place instrument on spot to be measured,
and hit [A-Z] to read white and setup FWA compensation (keyed to letter)
[a-z] to read and make FWA compensated reading from keyed reference
'r' to set reference, 's' to save spectrum,
'h' to toggle high res., 'k' to do a calibration
Hit ESC or Q to exit, any other key to take a reading:""",
                    """
 Result is XYZ: 25.849804 27.374348 27.163845, D50 Lab: 59.319238 -2.248166 -8.249728
                           CCT = 5996K (Delta E 5.653680)
 Closest Planckian temperature = 5804K (Delta E 4.968356)
 Closest Daylight temperature  = 5934K (Delta E 1.575347)

Place instrument on spot to be measured,
and hit [A-Z] to read white and setup FWA compensation (keyed to letter)
[a-z] to read and make FWA compensated reading from keyed reference
'r' to set reference, 's' to save spectrum,
'h' to toggle high res., 'k' to do a calibration
Hit ESC or Q to exit, any other key to take a reading:""",
                    """
 Result is XYZ: 5.898029 6.233309 6.427531, D50 Lab: 29.993614 -1.240610 -6.124217
                           CCT = 6197K (Delta E 4.937751)
 Closest Planckian temperature = 6011K (Delta E 4.350182)
 Closest Daylight temperature  = 6153K (Delta E 0.996499)

Place instrument on spot to be measured,
and hit [A-Z] to read white and setup FWA compensation (keyed to letter)
[a-z] to read and make FWA compensated reading from keyed reference
'r' to set reference, 's' to save spectrum,
'h' to toggle high res., 'k' to do a calibration
Hit ESC or Q to exit, any other key to take a reading:""",
                    """
 Result is XYZ: 0.104401 0.110705 0.109155, D50 Lab: 0.993614 -1.240610 -6.124217
                           CCT = 6197K (Delta E 4.937751)
 Closest Planckian temperature = 6011K (Delta E 4.350182)
 Closest Daylight temperature  = 6153K (Delta E 0.996499)

Place instrument on spot to be measured,
and hit [A-Z] to read white and setup FWA compensation (keyed to letter)
[a-z] to read and make FWA compensated reading from keyed reference
'r' to set reference, 's' to save spectrum,
'h' to toggle high res., 'k' to do a calibration
Hit ESC or Q to exit, any other key to take a reading:""",
                ],
                [
                    """
 Result is XYZ: 114.661874 122.077962 119.963424, D50 Lab: 107.975846 -4.649371 -12.841206
                           CCT = 5996K (Delta E 6.745175)
 Closest Planckian temperature = 5771K (Delta E 5.934859)
 Closest Daylight temperature  = 5898K (Delta E 2.538832)

Place instrument on spot to be measured,
and hit [A-Z] to read white and setup FWA compensation (keyed to letter)
[a-z] to read and make FWA compensated reading from keyed reference
'r' to set reference, 's' to save spectrum,
'h' to toggle high res., 'k' to do a calibration
Hit ESC or Q to exit, any other key to take a reading:""",
                    """
 Result is XYZ: 61.818899 65.859246 64.979340, D50 Lab: 84.924570 -3.876653 -10.701093
                           CCT = 6024K (Delta E 6.822442)
 Closest Planckian temperature = 5794K (Delta E 6.006530)
 Closest Daylight temperature  = 5922K (Delta E 2.615316)

Place instrument on spot to be measured,
and hit [A-Z] to read white and setup FWA compensation (keyed to letter)
[a-z] to read and make FWA compensated reading from keyed reference
'r' to set reference, 's' to save spectrum,
'h' to toggle high res., 'k' to do a calibration
Hit ESC or Q to exit, any other key to take a reading:""",
                    """
 Result is XYZ: 25.795743 27.295989 27.081138, D50 Lab: 59.247303 -2.163010 -8.233441
                           CCT = 5988K (Delta E 5.511791)
 Closest Planckian temperature = 5800K (Delta E 4.842103)
 Closest Daylight temperature  = 5931K (Delta E 1.447924)

Place instrument on spot to be measured,
and hit [A-Z] to read white and setup FWA compensation (keyed to letter)
[a-z] to read and make FWA compensated reading from keyed reference
'r' to set reference, 's' to save spectrum,
'h' to toggle high res., 'k' to do a calibration
Hit ESC or Q to exit, any other key to take a reading:""",
                    """
 Result is XYZ: 5.925222 6.263326 6.427544, D50 Lab: 30.067324 -1.256015 -5.997186
                           CCT = 6170K (Delta E 5.014894)
 Closest Planckian temperature = 5984K (Delta E 4.416741)
 Closest Daylight temperature  = 6124K (Delta E 1.057856)

Place instrument on spot to be measured,
and hit [A-Z] to read white and setup FWA compensation (keyed to letter)
[a-z] to read and make FWA compensated reading from keyed reference
'r' to set reference, 's' to save spectrum,
'h' to toggle high res., 'k' to do a calibration
Hit ESC or Q to exit, any other key to take a reading:""",
                    """
 Result is XYZ: 0.104401 0.110705 0.109155, D50 Lab: 0.067324 -1.256015 -5.997186
                           CCT = 6170K (Delta E 5.014894)
 Closest Planckian temperature = 5984K (Delta E 4.416741)
 Closest Daylight temperature  = 6124K (Delta E 1.057856)

Place instrument on spot to be measured,
and hit [A-Z] to read white and setup FWA compensation (keyed to letter)
[a-z] to read and make FWA compensated reading from keyed reference
'r' to set reference, 's' to save spectrum,
'h' to toggle high res., 'k' to do a calibration
Hit ESC or Q to exit, any other key to take a reading:""",
                ],
                [
                    """
 Result is XYZ: 116.839314 123.894968 122.159446, D50 Lab: 108.587904 -3.955352 -13.160232
                           CCT = 5975K (Delta E 5.963774)
 Closest Planckian temperature = 5774K (Delta E 5.240593)
 Closest Daylight temperature  = 5903K (Delta E 1.842768)

Place instrument on spot to be measured,
and hit [A-Z] to read white and setup FWA compensation (keyed to letter)
[a-z] to read and make FWA compensated reading from keyed reference
'r' to set reference, 's' to save spectrum,
'h' to toggle high res., 'k' to do a calibration
Hit ESC or Q to exit, any other key to take a reading:""",
                    """
 Result is XYZ: 63.469751 67.164836 66.177725, D50 Lab: 85.587118 -2.928291 -10.687360
                           CCT = 5951K (Delta E 5.590334)
 Closest Planckian temperature = 5764K (Delta E 4.908104)
 Closest Daylight temperature  = 5892K (Delta E 1.506948)

Place instrument on spot to be measured,
and hit [A-Z] to read white and setup FWA compensation (keyed to letter)
[a-z] to read and make FWA compensated reading from keyed reference
'r' to set reference, 's' to save spectrum,
'h' to toggle high res., 'k' to do a calibration
Hit ESC or Q to exit, any other key to take a reading:""",
                    """
 Result is XYZ: 26.106117 27.645671 27.403890, D50 Lab: 59.567264 -2.255152 -8.227727
                           CCT = 5991K (Delta E 5.663331)
 Closest Planckian temperature = 5798K (Delta E 4.976351)
 Closest Daylight temperature  = 5928K (Delta E 1.582241)

Place instrument on spot to be measured,
and hit [A-Z] to read white and setup FWA compensation (keyed to letter)
[a-z] to read and make FWA compensated reading from keyed reference
'r' to set reference, 's' to save spectrum,
'h' to toggle high res., 'k' to do a calibration
Hit ESC or Q to exit, any other key to take a reading:""",
                    """
 Result is XYZ: 6.001154 6.326623 6.508185, D50 Lab: 30.221991 -1.083417 -6.086282
                           CCT = 6157K (Delta E 4.496558)
 Closest Planckian temperature = 5990K (Delta E 3.957006)
 Closest Daylight temperature  = 6131K (Delta E 0.597640)

Place instrument on spot to be measured,
and hit [A-Z] to read white and setup FWA compensation (keyed to letter)
[a-z] to read and make FWA compensated reading from keyed reference
'r' to set reference, 's' to save spectrum,
'h' to toggle high res., 'k' to do a calibration
Hit ESC or Q to exit, any other key to take a reading:""",
                    """
 Result is XYZ: 0.104401 0.110705 0.109155, D50 Lab: 0.221991 -1.083417 -6.086282
                           CCT = 6157K (Delta E 4.496558)
 Closest Planckian temperature = 5990K (Delta E 3.957006)
 Closest Daylight temperature  = 6131K (Delta E 0.597640)

Place instrument on spot to be measured,
and hit [A-Z] to read white and setup FWA compensation (keyed to letter)
[a-z] to read and make FWA compensated reading from keyed reference
'r' to set reference, 's' to save spectrum,
'h' to toggle high res., 'k' to do a calibration
Hit ESC or Q to exit, any other key to take a reading:""",
                ],
            ][app.TopWindow.index][i]
            if i < 3:
                i += 1
            else:
                i -= 3
        elif bytes_ in ("Q", "q"):
            wx.CallAfter(app.TopWindow.Close)
            return
        else:
            return
        for line in txt.split("\n"):
            sleep(0.03125)
            wx.CallAfter(app.TopWindow.write, line)
            print(line)

    start_new_thread(test, tuple())
    app.MainLoop()



# -*- coding: utf-8 -*-
import os
from subprocess import Popen
import sys

from DisplayCAL import config
from DisplayCAL.argyll import (
    get_argyll_util,
    get_argyll_version,
    get_argyll_version_string,
)

from DisplayCAL.dev.mocks import check_call, check_call_str
from tests.data.argyll_sp_data import SUBPROCESS_COM


# todo: deactivated test temporarily
# def test_xicclu_is_working_properly(data_files):
#     """testing if ``DisplayCAL.worker_base.Xicclu`` is working properly"""
#     from DisplayCAL.icc_profile import ICCProfile
#     from DisplayCAL.worker_base import Xicclu
#
#     profile = ICCProfile(profile=data_files["default.icc"].absolute())
#     xicclu = Xicclu(profile, "r", "a", pcs="X", scale=100)
#     assert xicclu() is not None


def test_get_argyll_util(setup_argyll):
    """Test get_argyll_util() function."""
    config.initcfg()
    result = get_argyll_util("ccxxmake")
    expected_result = os.path.join(config.getcfg("argyll.dir"), "ccxxmake")
    if sys.platform == "win32":
        expected_result += ".exe"
    assert result == expected_result


def test_get_argyll_version_string_1(setup_argyll):
    """Test get_argyll_version_string() function."""
    config.initcfg()
    with check_call(Popen, "communicate", SUBPROCESS_COM):
        result = get_argyll_version_string("ccxxmake")
    expected_result = "2.3.0"
    assert result == expected_result


def test_get_argyll_version_1(setup_argyll):
    """Test get_argyll_version() function."""
    with check_call_str("DisplayCAL.argyll.get_argyll_version_string", "2.3.0"):
        result = get_argyll_version("ccxxmake")
    expected_result = [2, 3, 0]
    assert result == expected_result
